/*====================================================================
| Version: December 8, 2010
\===================================================================*/

/*====================================================================
| Ricard Delgado-Gonzalo
| EPFL/STI/IOA/LIB/BM.4.141
| e-mail: ricard.delgado@gmail.com
| URL: http://www.ricarddelgado.com/
\===================================================================*/
package analysis;

import filters.CTLoG;
import filters.CTQuadraticSteerableJFilter;
import ij.process.FloatProcessor;

import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.geom.Point2D;
import java.util.Arrays;
import java.util.Vector;

public class CTCellAnalyzer {

	public static double getCellSteerable4Energy(Polygon cell, double cellArea, FloatProcessor imageData, int regionXmin, int regionYmin, int regionWidth, int regionHeight, double sigma){
		double energy = 0.0;
		if(Math.round(8*sigma)<=regionWidth && Math.round(8*sigma)<=regionHeight){
			CTQuadraticSteerableJFilter filterSteerable4 = new CTQuadraticSteerableJFilter(imageData, sigma);
			filterSteerable4.filter();
			double[] filteredImage = filterSteerable4.getNMS();
			Rectangle bounds = cell.getBounds();
			int xMin = (int)Math.max(regionXmin, bounds.getMinX());
			int yMin = (int)Math.max(regionYmin, bounds.getMinY());
			int xMax = (int)Math.min(regionXmin+regionWidth-1, bounds.getMaxX());
			int yMax = (int)Math.min(regionYmin+regionHeight-1, bounds.getMaxY());
			for(int x=xMin; x<=xMax; x++){
				for(int y=yMin; y<=yMax; y++){
					if(cell.contains(x, y)){
						double value = filteredImage[(x-regionXmin)+regionWidth*(y-regionYmin)];
						energy += value;
					}
				}
			}
		}
		return energy;
	}
	
	
	public static double getCellLoGEnergy(Polygon cell, double cellArea, FloatProcessor imageData, int regionXmin, int regionYmin, int regionWidth, int regionHeight, double std, double th){
		FloatProcessor filtered = (FloatProcessor)imageData.duplicate();
		CTLoG filterLoG = new CTLoG();
		filterLoG.runFilter(filtered, std, std);
		float[] imageDataArray = (float[])filtered.getPixels();
			
		//compute energy mean		
		double energy = 0.0;
		Rectangle bounds = cell.getBounds();
		int xMin = (int)Math.max(regionXmin, bounds.getMinX());
		int yMin = (int)Math.max(regionYmin, bounds.getMinY());
		int xMax = (int)Math.min(regionXmin+regionWidth-1, bounds.getMaxX());
		int yMax = (int)Math.min(regionYmin+regionHeight-1, bounds.getMaxY());
		for(int x=xMin; x<=xMax; x++){
			for(int y=yMin; y<=yMax; y++){
				if(cell.contains(x, y)){
					double value = imageDataArray[(x-regionXmin)+regionWidth*(y-regionYmin)];
					if(value>=th){
						energy += value;
					}
				}
			}
		}
		return energy;
	}
	
	
	public static double getCellTopNMean(Polygon cell, double cellArea, FloatProcessor imageData, int regionXmin, int regionYmin, int regionWidth, int regionHeight, int n){
		if(cellArea>0.0 && n>0){
			int arrayLength = (int)Math.ceil(cellArea);
			double[] pixArray = new double[arrayLength];
			float[] imageDataArray = (float[])imageData.getPixels();
			Rectangle bounds = cell.getBounds();
			int xMin = (int)Math.max(regionXmin, bounds.getMinX());
			int yMin = (int)Math.max(regionYmin, bounds.getMinY());
			int xMax = (int)Math.min(regionXmin+regionWidth-1, bounds.getMaxX());
			int yMax = (int)Math.min(regionYmin+regionHeight-1, bounds.getMaxY());
			int count = 0;
			for(int x=xMin; x<=xMax; x++){
				for(int y=yMin; y<=yMax; y++){
					if(cell.contains(x, y)){
						pixArray[count] = imageDataArray[(x-regionXmin)+regionWidth*(y-regionYmin)];
						count++;
					}
				}
			}
			Arrays.sort(pixArray);
			double topNMean = 0.0;
			int topNCont = 0;

			for(int i=arrayLength-1; i>=0 && i>=(arrayLength-n); i--){
				topNCont++;
				topNMean += pixArray[i]; 
			}
			
			if(topNCont>0){
				return (topNMean/(double)topNCont);
			}
		}
		return 0.0;
	}
	
	public static double computeCircularity(Polygon cell, double cellArea) {
		double P = 0;
		int npoints   = cell.npoints;
		int[] xpoints = cell.xpoints;
		int[] ypoints = cell.ypoints;
		for(int point=0; point<npoints; point++){
			double dx = (xpoints[(point+1)%npoints]-xpoints[point]);
			double dy = (ypoints[(point+1)%npoints]-ypoints[point]);
			P += Math.sqrt(dx*dx+dy*dy);
		}
		return 4*Math.PI*cellArea/(P*P);
	}
	
	public static double getCellAreaVSConvexHullArea(Polygon cell, double cellArea){
		if(cellArea>0.0){
			Vector<Point2D.Double> cellOutline = new Vector<Point2D.Double>();
			int nPoints = cell.npoints;
			int xPoints[] = cell.xpoints;
			int yPoints[] = cell.ypoints;
			
			for(int i=0; i<nPoints; i++){
				cellOutline.add(new Point2D.Double((double)xPoints[i],(double)yPoints[i]));
			}
			
			CTMelkman melkman = new CTMelkman(cellOutline);
			Vector<Point2D.Double> convexHullOutlineVector = melkman.getOutputPoints();
			
			int nPointsConvexHull = convexHullOutlineVector.size();
			int xPointsConvexHull[] = new int[nPointsConvexHull];
			int yPointsConvexHull[] = new int[nPointsConvexHull];
			
			for(int i=0; i<nPointsConvexHull; i++){
				xPointsConvexHull[i] = 	(int) Math.round(convexHullOutlineVector.elementAt(i).x);
				yPointsConvexHull[i] = 	(int) Math.round(convexHullOutlineVector.elementAt(i).y);
			}
			
			Polygon convexHullOutline = new Polygon(xPointsConvexHull, yPointsConvexHull, nPointsConvexHull);
			Rectangle bounds = convexHullOutline.getBounds();
			int convexHullArea = 0;
			for(int x=(int) bounds.getMinX(); x<=bounds.getMaxX(); x++){
				for(int y=(int) bounds.getMinY(); y<=bounds.getMaxY(); y++){
					if(convexHullOutline.contains(x, y)){
						convexHullArea++;
					}
				}
			}
			if(convexHullArea>0){
				return (double)cellArea/(double)convexHullArea;
			}
		}
		return 0.0;
	}
	
	public static double getCellArea(Polygon cell){
		int cellArea = 0;
		Rectangle bounds = cell.getBounds();
		for(int x=(int) bounds.getMinX(); x<=bounds.getMaxX(); x++){
			for(int y=(int) bounds.getMinY(); y<=bounds.getMaxY(); y++){
				if(cell.contains(x, y)){
					cellArea++;
				}
			}
		}
		return (double)cellArea;
	}
	
	public static double getCellMedian(Polygon cell, double cellArea, FloatProcessor imageData, int regionXmin, int regionYmin, int regionWidth, int regionHeight){
		if(cellArea>0.0){
			double[] pixArray = new double[(int)Math.ceil(cellArea)];
			float[] imageDataArray = (float[])imageData.getPixels();
			Rectangle bounds = cell.getBounds();
			int xMin = (int)Math.max(regionXmin, bounds.getMinX());
			int yMin = (int)Math.max(regionYmin, bounds.getMinY());
			int xMax = (int)Math.min(regionXmin+regionWidth-1, bounds.getMaxX());
			int yMax = (int)Math.min(regionYmin+regionHeight-1, bounds.getMaxY());
			int count = 0;
			for(int x=xMin; x<=xMax; x++){
				for(int y=yMin; y<=yMax; y++){
					if(cell.contains(x, y)){
						pixArray[count] = imageDataArray[(x-regionXmin)+regionWidth*(y-regionYmin)];
						count++;
					}
				}
			}
			Arrays.sort(pixArray);
			int medianPix = (int)Math.ceil(cellArea/2.0)-1;
			if(medianPix>=0){
				return pixArray[medianPix];
			}else{
				return pixArray[0];
			}
		}
		return 0.0;
	}
	
	public static double getCellMean(Polygon cell, double cellArea, FloatProcessor imageData, int regionXmin, int regionYmin, int regionWidth, int regionHeight){
		if(cellArea>0.0){
			float[] imageDataArray = (float[])imageData.getPixels();
			double cellMean = 0;
			Rectangle bounds = cell.getBounds();
			int xMin = (int)Math.max(regionXmin, bounds.getMinX());
			int yMin = (int)Math.max(regionYmin, bounds.getMinY());
			int xMax = (int)Math.min(regionXmin+regionWidth-1, bounds.getMaxX());
			int yMax = (int)Math.min(regionYmin+regionHeight-1, bounds.getMaxY());
			for(int x=xMin; x<=xMax; x++){
				for(int y=yMin; y<=yMax; y++){
					if(cell.contains(x, y)){
						cellMean +=imageDataArray[(x-regionXmin)+regionWidth*(y-regionYmin)];
					}
				}
			}
			return (double)cellMean/(double)cellArea;
		}
		return 0.0;
	}
	
	
	
	public static double getRegionNotCellBottomNPercentValue(Polygon cell, int region, double percent, int regionXmin, int regionYmin, int regionWidth, int regionHeight, double[] rawData, double[] watershedRegionMap){
		if(percent<0.0 || percent>100.0){
			return 0.0;
		}
		int NBackgroundPix = 0;
		for(int x=0; x<regionWidth; x++){
			for(int y=0; y<regionHeight; y++){
				if((region+11)==(int)Math.round(watershedRegionMap[x+regionWidth*y])){
					if(!cell.contains(x+regionXmin, y+regionYmin)){
						NBackgroundPix++;
					}
				}
			}
		}
		if(NBackgroundPix>0){
			double[] pixArray = new double[NBackgroundPix];
			int count = 0;
			for(int x=0; x<regionWidth; x++){
				for(int y=0; y<regionHeight; y++){
					if((region+11)==(int)Math.round(watershedRegionMap[x+regionWidth*y])){
						if(!cell.contains(x+regionXmin, y+regionYmin)){
							pixArray[count] = rawData[x+y*regionWidth];
							count++;
						}
					}
				}
			}
			Arrays.sort(pixArray);
			
			int nPixPercent = (int)Math.ceil(percent*(double)NBackgroundPix/100.0)-1;
			if(nPixPercent>=0){
				return(pixArray[nPixPercent]);
			}else{
				return(pixArray[0]);
			}
		}
		return 0.0;
	}


	public static double getRegionBottomNPercentValue(int region, double percent, int regionXmin, int regionYmin, int regionWidth, int regionHeight, double[] rawData, double[] watershedRegionMap){
		if(percent<0.0 || percent>100.0){
			return 0.0;
		}
		int NBackgroundPix = 0;
		for(int x=0; x<regionWidth; x++){
			for(int y=0; y<regionHeight; y++){
				if((region+11)==(int)Math.round(watershedRegionMap[x+regionWidth*y])){
					NBackgroundPix++;
				}
			}
		}
		if(NBackgroundPix>0){
			double[] pixArray = new double[NBackgroundPix];
			int count = 0;
			for(int x=0; x<regionWidth; x++){
				for(int y=0; y<regionHeight; y++){
					if((region+11)==(int)Math.round(watershedRegionMap[x+regionWidth*y])){
						pixArray[count] = rawData[x+y*regionWidth];
						count++;
					}
				}
			}
			Arrays.sort(pixArray);
			
			int nPixPercent = (int)Math.ceil(percent*(double)NBackgroundPix/100.0)-1;
			if(nPixPercent>=0){
				return(pixArray[nPixPercent]);
			}else{
				return(pixArray[0]);
			}
		}
		return 0.0;
	}

	
	
	public static double getMinValueRegion(int region, int regionXmin, int regionYmin, int regionWidth, int regionHeight, double[] rawData, double[] watershedRegionMap){
		double min = Double.MAX_VALUE;
		for(int x=0; x<regionWidth; x++){
			for(int y=0; y<regionHeight; y++){
				if((region+11)==(int)Math.round(watershedRegionMap[x+regionWidth*y])){
					double value = rawData[x+y*regionWidth];
					if(value<min){
						min = value;
					}
				}
			}
		}
		if(min == Double.MAX_VALUE){
			return 0.0;
		}else{
			return min;
		}
	}

	public static double getMinValueRegionNotCell(Polygon cell, int region, int regionXmin, int regionYmin, int regionWidth, int regionHeight, double[] rawData, double[] watershedRegionMap){
		double min = Double.MAX_VALUE;
		for(int x=0; x<regionWidth; x++){
			for(int y=0; y<regionHeight; y++){
				if((region+11)==(int)Math.round(watershedRegionMap[x+regionWidth*y])){
					if(!cell.contains(x+regionXmin, y+regionYmin)){
						double value = rawData[x+y*regionWidth];
						if(value<min){
							min = value;
						}
					}
				}
			}
		}
		if(min == Double.MAX_VALUE){
			return 0.0;
		}else{
			return min;
		}
	}
	
	
	public static double getRegionNotCellMeanVSCellMean(Polygon cell, double cellMean, int region, int regionXmin, int regionYmin, int regionWidth, int regionHeight, double[] rawData, double[] watershedRegionMap) {
		if(cellMean!=0.0){
			int NBackgroundPix = 0;
			double regionNotCellMean = 0;
			for(int x=0; x<regionWidth; x++){
				for(int y=0; y<regionHeight; y++){
					if((region+11)==(int)Math.round(watershedRegionMap[x+regionWidth*y])){
						if(!cell.contains(x+regionXmin, y+regionYmin)){
							regionNotCellMean += rawData[x+y*regionWidth];
							NBackgroundPix++;
						}
					}
				}
			}
			if(NBackgroundPix>1){
				regionNotCellMean /= (double)(NBackgroundPix-1);
				return(regionNotCellMean/cellMean);
			}else if(NBackgroundPix==1){
				return(regionNotCellMean/cellMean);
			}
		}
		return 0.0;
	}
	
	public static double getCellMeanVScellStd(Polygon cell, double cellArea, double cellMean, FloatProcessor imageData, int regionXmin, int regionYmin, int regionWidth, int regionHeight){
		if(cellArea>0.0 && cellMean!=0.0){
			float[] imageDataArray = (float[])imageData.getPixels();
			double cellStd = 0;
			Rectangle bounds = cell.getBounds();
			int xMin = (int)Math.max(regionXmin, bounds.getMinX());
			int yMin = (int)Math.max(regionYmin, bounds.getMinY());
			int xMax = (int)Math.min(regionXmin+regionWidth-1, bounds.getMaxX());
			int yMax = (int)Math.min(regionYmin+regionHeight-1, bounds.getMaxY());
			for(int x=xMin; x<=xMax; x++){
				for(int y=yMin; y<=yMax; y++){
					if(cell.contains(x, y)){
						cellStd += (imageDataArray[(x-regionXmin)+regionWidth*(y-regionYmin)]-cellMean);
					}
				}
			}
			if(cellArea>1.0){
				cellStd /= (cellArea-1);
				return(cellStd/cellMean); 
			}else if(cellArea>0.0){
				cellStd /= cellArea;
				return(cellStd/cellMean); 
			}
		}
		return 0.0;
	}
}
