/*====================================================================
| Version: December 17, 2010
\===================================================================*/

/*====================================================================
| Ricard Delgado-Gonzalo
| EPFL/STI/IOA/LIB/BM.4.141
| e-mail: ricard.delgado@gmail.com
| URL: http://www.ricarddelgado.com/
\===================================================================*/
package io;

import ij.IJ;
import ij.ImagePlus;
import ij.gui.Roi;
import ij.io.Opener;
import ij.process.FloatProcessor;
import ij.process.ImageProcessor;

import java.awt.Polygon;
import java.awt.Rectangle;
import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import structures.CTConnectivity;
import structures.CTExtractedData;
import structures.CTRunParameters;

public class CTInput {
	
	public static double[] loadParamXML(String absolutePath) {
		File f = new File(absolutePath);
		if (f.exists()) { // else create on exit			
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			try {
				DocumentBuilder db = dbf.newDocumentBuilder();
				Document dom = db.parse(f);
				
	            // normalize text representation
	            dom.getDocumentElement().normalize();
	            
	            Element docEle = dom.getDocumentElement();
	            double[] par = new double[2]; 
	            par[0] = Double.parseDouble(((Element)docEle.getElementsByTagName("LoGTH").item(0)).getTextContent());
	            par[1] = Double.parseDouble(((Element)docEle.getElementsByTagName("steerableTH").item(0)).getTextContent());
	            return(par);
			}catch(ParserConfigurationException pce) {
				IJ.error(""+pce);
			}catch(SAXException se) {
				IJ.error(""+se);
			}catch(IOException ioe) {
				IJ.error(""+ioe);
			}
		}
		return null;
	}
	
	static public String[] getListDir(String path) {
		File dir = new File(path);
		if (!dir.isDirectory()) {
			IJ.error("Error","Directory Error");
			return null;
		}
		String[] list1 = dir.list();
		int count = 0;
		for(int i=0; i<list1.length; i++) {
			if (!list1[i].startsWith("."))
				count++;
		}
		String[] list = new String[count];
		count = 0;
		for(int i=0; i<list1.length; i++) {
			if (!list1[i].startsWith("."))
				list[count++] = list1[i];	
		}	
		return list;
	}

	static public FloatProcessor readProcessor(String path, String file, Roi roi){
		Opener opener = new Opener();
		ImagePlus imageplus = opener.openImage(path,file);
		
		if(roi!=null){
			imageplus.setRoi(roi);
			ImageProcessor flpr = imageplus.getProcessor().crop();
			return flpr.toFloat(0, null);
		}else{
			ImageProcessor flpr = imageplus.getProcessor();
			return flpr.toFloat(0, null);
		}
		
	}

	public static boolean loadXML(String absolutePath,	CTRunParameters runParameters, CTExtractedData[] extractedData, CTConnectivity[] linkedData) {
		Rectangle roiBox = null;
		double lambda;
		double sigma;
		double sigmaNotMovingRegion;
		int regionOffset;
		double downDisplacement;
		double upDisplacement;
		double lateralDisplacement;
		int regularizationIter;
		
		File f = new File(absolutePath);
		if (f.exists()) { // else create on exit			
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			try {
				DocumentBuilder db = dbf.newDocumentBuilder();
				Document dom = db.parse(f);
				
	            // normalize text representation
	            dom.getDocumentElement().normalize();
	            
	            Element docEle = dom.getDocumentElement();
	            int xMin = Integer.parseInt(((Element)docEle.getElementsByTagName("xMin").item(0)).getTextContent());
	            int yMin = Integer.parseInt(((Element)docEle.getElementsByTagName("yMin").item(0)).getTextContent());
	            int width = Integer.parseInt(((Element)docEle.getElementsByTagName("W").item(0)).getTextContent());
	            int height = Integer.parseInt(((Element)docEle.getElementsByTagName("H").item(0)).getTextContent());
	            roiBox = new Rectangle(xMin, yMin, width, height);
	            
	            int Nstack = Integer.parseInt(((Element)docEle.getElementsByTagName("Nim").item(0)).getTextContent());
	            
	            if(Nstack!=extractedData.length){
	            	return false;
	            }
	            
	            regularizationIter = Integer.parseInt(((Element)docEle.getElementsByTagName("iter").item(0)).getTextContent());
	            lambda = Double.parseDouble(((Element)docEle.getElementsByTagName("lambda").item(0)).getTextContent());
	            sigma = Double.parseDouble(((Element)docEle.getElementsByTagName("sigma").item(0)).getTextContent());
	            sigmaNotMovingRegion = Double.parseDouble(((Element)docEle.getElementsByTagName("NMR").item(0)).getTextContent());
	            regionOffset = Integer.parseInt(((Element)docEle.getElementsByTagName("offset").item(0)).getTextContent());
	            upDisplacement = Double.parseDouble(((Element)docEle.getElementsByTagName("upD").item(0)).getTextContent());
	            downDisplacement = Double.parseDouble(((Element)docEle.getElementsByTagName("downD").item(0)).getTextContent());
	            lateralDisplacement = Double.parseDouble(((Element)docEle.getElementsByTagName("latD").item(0)).getTextContent());

	            NodeList frameStackList = docEle.getElementsByTagName("frame");
	            
	            for(int i=0; i<Nstack; i++){
		            Element frameStackElement = (Element)frameStackList.item(i);
            		int Nregions = Integer.parseInt(getTextValue(frameStackElement,"Nreg"));
            		extractedData[i] = new CTExtractedData();
	            	linkedData[i] = new CTConnectivity(Nregions);
	            	linkedData[i].frame = i;
	            	
    				double[] regionCentroidX 						= new double[Nregions];
    				double[] regionCentroidY 						= new double[Nregions];
    				double[] regionArea 							= new double[Nregions];
    				double[] cellArea 								= new double[Nregions];
    				int[] regionXmin 								= new int[Nregions];
    				int[] regionXmax 								= new int[Nregions];
    				int[] regionYmin 								= new int[Nregions];
    				int[] regionYmax 								= new int[Nregions];
    				double[] regionMeanCh1 							= new double[Nregions];
    				double[] regionMeanCh2 							= new double[Nregions];
    				double[] cellMeanCh1							= new double[Nregions];
    				double[] cellMeanCh2							= new double[Nregions];
    				double[] cellTop3MeanCh1						= new double[Nregions];
    				double[] cellTop3MeanCh2						= new double[Nregions];
    				double[] cellTop5MeanCh1						= new double[Nregions];
    				double[] cellTop5MeanCh2						= new double[Nregions];
    				double[] cellMedianCh1							= new double[Nregions];
    				double[] cellMedianCh2							= new double[Nregions];
    				double[] minValueRegionCh1 						= new double[Nregions];
    				double[] minValueRegionCh2 						= new double[Nregions];
    				double[] minValueRegionNotCellCh1 				= new double[Nregions];
    				double[] minValueRegionNotCellCh2 				= new double[Nregions];
    				double[] regionBottomNPercentValueCh1 			= new double[Nregions];
    				double[] regionBottomNPercentValueCh2 			= new double[Nregions];
    				double[] regionNotCellBottomNPercentValueCh1	= new double[Nregions];
    				double[] regionNotCellBottomNPercentValueCh2 	= new double[Nregions];
    				int[] segmentationType 							= new int[Nregions];
    				double[] segmentationQuality 					= new double[Nregions];
    				double[] cellCircularity 						= new double[Nregions];
    				double[] cellMeanVScellStdCh1					= new double[Nregions];
    				double[] cellMeanVScellStdCh2					= new double[Nregions];
    				double[] regionNotCellMeanVSCellMeanCh1			= new double[Nregions];
    				double[] regionNotCellMeanVSCellMeanCh2			= new double[Nregions];
    				double[] energyLoG10Ch1							= new double[Nregions];
    				double[] energyLoG10Ch2							= new double[Nregions];
    				double[] energyLoG3Ch1							= new double[Nregions];
    				double[] energyLoG3Ch2							= new double[Nregions];
    				double[] energySteerable4Ch1					= new double[Nregions];
    				double[] energySteerable4Ch2					= new double[Nregions];
    				
    				Polygon[] cellOutline 						= new Polygon[Nregions];
    				NodeList centroidsX 						= frameStackElement.getElementsByTagName("rXc");
    				NodeList centroidsY 						= frameStackElement.getElementsByTagName("rYc");
    				NodeList areaRegions						= frameStackElement.getElementsByTagName("rArea");
    				NodeList xmins 								= frameStackElement.getElementsByTagName("rXmin");
    				NodeList xmaxs 								= frameStackElement.getElementsByTagName("rXmax");
    				NodeList ymins 								= frameStackElement.getElementsByTagName("rYmin");
    				NodeList ymaxs 								= frameStackElement.getElementsByTagName("rYmax");
    				NodeList rmeans1							= frameStackElement.getElementsByTagName("rMean1");
    				NodeList rmeans2		 					= frameStackElement.getElementsByTagName("rMean2");
    				NodeList rMins1								= frameStackElement.getElementsByTagName("rMin1");
    				NodeList rMins2		 						= frameStackElement.getElementsByTagName("rMin2");
    				NodeList rNotCellMin1						= frameStackElement.getElementsByTagName("rNotCellMin1");
    				NodeList rNotCellMin2		 				= frameStackElement.getElementsByTagName("rNotCellMin2");
    				NodeList rBot5per1							= frameStackElement.getElementsByTagName("rBot5per1");
    				NodeList rBot5per2		 					= frameStackElement.getElementsByTagName("rBot5per2");
    				NodeList rNotCellBot5per1					= frameStackElement.getElementsByTagName("rNotCellBot5per1");
    				NodeList rNotCellBot5per2					= frameStackElement.getElementsByTagName("rNotCellBot5per2");
    				NodeList means1								= frameStackElement.getElementsByTagName("mean1");
    				NodeList means2		 						= frameStackElement.getElementsByTagName("mean2");
    				NodeList Top3means1							= frameStackElement.getElementsByTagName("Top3mean1");
    				NodeList Top3means2		 					= frameStackElement.getElementsByTagName("Top3mean2");
    				NodeList Top5means1							= frameStackElement.getElementsByTagName("Top5mean1");
    				NodeList Top5means2		 					= frameStackElement.getElementsByTagName("Top5mean2");
    				NodeList medians1							= frameStackElement.getElementsByTagName("median1");
    				NodeList medians2	 						= frameStackElement.getElementsByTagName("median2");
    				NodeList areas		 						= frameStackElement.getElementsByTagName("area");
    				NodeList segmentationTypes					= frameStackElement.getElementsByTagName("type");
    				NodeList segmentationQualities				= frameStackElement.getElementsByTagName("qos");
    				NodeList cellCircularities					= frameStackElement.getElementsByTagName("circ");
    				NodeList cellMeanVScellStdCh1s				= frameStackElement.getElementsByTagName("meanVSstd1");
    				NodeList cellMeanVScellStdCh2s				= frameStackElement.getElementsByTagName("meanVSstd2");
    				NodeList regionNotCellMeanVSCellMeanCh1s	= frameStackElement.getElementsByTagName("meanNotCellVSCell1");
    				NodeList regionNotCellMeanVSCellMeanCh2s	= frameStackElement.getElementsByTagName("meanNotCellVSCell2");
    				NodeList Npols								= frameStackElement.getElementsByTagName("Npol");
    				NodeList xs									= frameStackElement.getElementsByTagName("x");
    				NodeList ys									= frameStackElement.getElementsByTagName("y");
    	            NodeList trackLabels						= frameStackElement.getElementsByTagName("track");
    				NodeList mothers 							= frameStackElement.getElementsByTagName("mother");
    				NodeList Nchildrens 						= frameStackElement.getElementsByTagName("Nch");
    				NodeList daugters 							= frameStackElement.getElementsByTagName("child");
    				NodeList LoG10ch1s 							= frameStackElement.getElementsByTagName("LoG10ch1");
    				NodeList LoG10ch2s 							= frameStackElement.getElementsByTagName("LoG10ch2");
    				NodeList LoG3ch1s 							= frameStackElement.getElementsByTagName("LoG3ch1");
    				NodeList LoG3ch2s 							= frameStackElement.getElementsByTagName("LoG3ch2");
    				NodeList steerable4Ch1s 					= frameStackElement.getElementsByTagName("steerable4ch1");
    				NodeList steerable4Ch2s 					= frameStackElement.getElementsByTagName("steerable4ch2");
    				
    				int daugtercont = 0;
    				for(int j=0; j<Nregions; j++) {
    					Element centroidXe = (Element)centroidsX.item(j);
    					regionCentroidX[j] = Double.parseDouble(centroidXe.getFirstChild().getNodeValue());
    					
    					Element centroidYe = (Element)centroidsY.item(j);
    					regionCentroidY[j] = Double.parseDouble(centroidYe.getFirstChild().getNodeValue());
    					
    					Element areaRegionse = (Element)areaRegions.item(j);
    					regionArea[j] = Double.parseDouble(areaRegionse.getFirstChild().getNodeValue());
    					
    					Element xmine = (Element)xmins.item(j);
    					regionXmin[j] = Integer.parseInt(xmine.getFirstChild().getNodeValue());
    					
    					Element xmaxe = (Element)xmaxs.item(j);
    					regionXmax[j] = Integer.parseInt(xmaxe.getFirstChild().getNodeValue());
    					
    					Element ymine = (Element)ymins.item(j);
    					regionYmin[j] = Integer.parseInt(ymine.getFirstChild().getNodeValue());
    					
    					Element ymaxe = (Element)ymaxs.item(j);
    					regionYmax[j] = Integer.parseInt(ymaxe.getFirstChild().getNodeValue());
    					
    					Element regionMeanCh1e = (Element)rmeans1.item(j);
    					regionMeanCh1[j] = Double.parseDouble(regionMeanCh1e.getFirstChild().getNodeValue());
    					
    					Element regionMeanCh2e = (Element)rmeans2.item(j);
    					regionMeanCh2[j] = Double.parseDouble(regionMeanCh2e.getFirstChild().getNodeValue());
    					
    					Element rMins1e = (Element)rMins1.item(j);
    					minValueRegionCh1[j] = Double.parseDouble(rMins1e.getFirstChild().getNodeValue());
    					
    					Element rMins2e = (Element)rMins2.item(j);
    					minValueRegionCh2[j] = Double.parseDouble(rMins2e.getFirstChild().getNodeValue());
    					
    					Element rNotCellMin1e = (Element)rNotCellMin1.item(j);
    					minValueRegionNotCellCh1[j] = Double.parseDouble(rNotCellMin1e.getFirstChild().getNodeValue());
    					
    					Element rNotCellMin2e = (Element)rNotCellMin2.item(j);
    					minValueRegionNotCellCh2[j] = Double.parseDouble(rNotCellMin2e.getFirstChild().getNodeValue());
    					
    					Element rBot5per1e = (Element)rBot5per1.item(j);
    					regionBottomNPercentValueCh1[j] = Double.parseDouble(rBot5per1e.getFirstChild().getNodeValue());
    					
    					Element rBot5per2e = (Element)rBot5per2.item(j);
    					regionBottomNPercentValueCh2[j] = Double.parseDouble(rBot5per2e.getFirstChild().getNodeValue());
    					
    					Element rNotCellBot5per1e = (Element)rNotCellBot5per1.item(j);
    					regionNotCellBottomNPercentValueCh1[j] = Double.parseDouble(rNotCellBot5per1e.getFirstChild().getNodeValue());
    					
    					Element rNotCellBot5per2e = (Element)rNotCellBot5per2.item(j);
    					regionNotCellBottomNPercentValueCh2[j] = Double.parseDouble(rNotCellBot5per2e.getFirstChild().getNodeValue());
    					
    					Element cellMeanCh1e = (Element)means1.item(j);
    					cellMeanCh1[j] = Double.parseDouble(cellMeanCh1e.getFirstChild().getNodeValue());
    					
    					Element cellMeanCh2e = (Element)means2.item(j);
    					cellMeanCh2[j] = Double.parseDouble(cellMeanCh2e.getFirstChild().getNodeValue());
    					
    					Element cellTop3MeanCh1e = (Element)Top3means1.item(j);
    					cellTop3MeanCh1[j] = Double.parseDouble(cellTop3MeanCh1e.getFirstChild().getNodeValue());
    					
    					Element cellTop3MeanCh2e = (Element)Top3means2.item(j);
    					cellTop3MeanCh2[j] = Double.parseDouble(cellTop3MeanCh2e.getFirstChild().getNodeValue());
    					
    					Element cellTop5MeanCh1e = (Element)Top5means1.item(j);
    					cellTop5MeanCh1[j] = Double.parseDouble(cellTop5MeanCh1e.getFirstChild().getNodeValue());
    					
    					Element cellTop5MeanCh2e = (Element)Top5means2.item(j);
    					cellTop5MeanCh2[j] = Double.parseDouble(cellTop5MeanCh2e.getFirstChild().getNodeValue());
    				
    					Element cellMedianCh1e = (Element)medians1.item(j);
    					cellMedianCh1[j] = Double.parseDouble(cellMedianCh1e.getFirstChild().getNodeValue());
    					
    					Element cellMedianCh2e = (Element)medians2.item(j);
    					cellMedianCh2[j] = Double.parseDouble(cellMedianCh2e.getFirstChild().getNodeValue());
    					 
    					Element arease = (Element)areas.item(j);
    					cellArea[j] = Double.parseDouble(arease.getFirstChild().getNodeValue());
    					
    					Element segmentationTypese = (Element)segmentationTypes.item(j);
    					String type = segmentationTypese.getFirstChild().getNodeValue();
    					
    					if(type.equalsIgnoreCase("OVUSCULE")){
    						segmentationType[j] = CTExtractedData.OVUSCULE;
    					}else if(type.equalsIgnoreCase("REGION")){
    						segmentationType[j] = CTExtractedData.REGION;
    					}else if(type.equalsIgnoreCase("ESNAKE")){
    						segmentationType[j] = CTExtractedData.ESNAKE;
    					}
    					
    					Element segmentationQualitye = (Element)segmentationQualities.item(j);
    					segmentationQuality[j] = Double.parseDouble(segmentationQualitye.getFirstChild().getNodeValue());
    					
    					Element cellCircularitye = (Element)cellCircularities.item(j);
    					cellCircularity[j] = Double.parseDouble(cellCircularitye.getFirstChild().getNodeValue());
    					
    					Element cellMeanVScellStdCh1e = (Element)cellMeanVScellStdCh1s.item(j);
    					cellMeanVScellStdCh1[j] = Double.parseDouble(cellMeanVScellStdCh1e.getFirstChild().getNodeValue());
    					
    					Element cellMeanVScellStdCh2e = (Element)cellMeanVScellStdCh2s.item(j);
    					cellMeanVScellStdCh2[j] = Double.parseDouble(cellMeanVScellStdCh2e.getFirstChild().getNodeValue());
    					
    					Element regionNotCellMeanVSCellMeanCh1e = (Element)regionNotCellMeanVSCellMeanCh1s.item(j);
    					regionNotCellMeanVSCellMeanCh1[j] = Double.parseDouble(regionNotCellMeanVSCellMeanCh1e.getFirstChild().getNodeValue());
    					
    					Element regionNotCellMeanVSCellMeanCh2e = (Element)regionNotCellMeanVSCellMeanCh2s.item(j);
    					regionNotCellMeanVSCellMeanCh2[j] = Double.parseDouble(regionNotCellMeanVSCellMeanCh2e.getFirstChild().getNodeValue());
    					
    					Element LoG10ch1e = (Element)LoG10ch1s.item(j);
    					energyLoG10Ch1[j] = Double.parseDouble(LoG10ch1e.getFirstChild().getNodeValue());
    					
    					Element LoG10ch2e = (Element)LoG10ch2s.item(j);
    					energyLoG10Ch2[j] = Double.parseDouble(LoG10ch2e.getFirstChild().getNodeValue());

    					Element LoG3ch1e = (Element)LoG3ch1s.item(j);
    					energyLoG3Ch1[j] = Double.parseDouble(LoG3ch1e.getFirstChild().getNodeValue());
    					
    					Element LoG3ch2e = (Element)LoG3ch2s.item(j);
    					energyLoG3Ch2[j] = Double.parseDouble(LoG3ch2e.getFirstChild().getNodeValue());
    					
    					Element steerable4Ch1e = (Element)steerable4Ch1s.item(j);
    					energySteerable4Ch1[j] = Double.parseDouble(steerable4Ch1e.getFirstChild().getNodeValue());
    					
    					Element steerable4Ch2e = (Element)steerable4Ch2s.item(j);
    					energySteerable4Ch2[j] = Double.parseDouble(steerable4Ch2e.getFirstChild().getNodeValue());
    					
    					Element Npole = (Element)Npols.item(j);
    					int npoints = Integer.parseInt(Npole.getFirstChild().getNodeValue());

    					Element xe = (Element)xs.item(j);
    					String x = xe.getFirstChild().getNodeValue();
    					int[] xvec = new int[npoints];

    					String[] xSplit = x.split("\t");
    					for(int n=0; n<npoints; n++){
    						xvec[n] = Integer.parseInt(xSplit[n]);
						}
    					
    					Element ye = (Element)ys.item(j);
    					String y = ye.getFirstChild().getNodeValue();
    					int[] yvec = new int[npoints];
    					
    					String[] ySplit = y.split("\t");
    					for(int n=0; n<npoints; n++){
    						yvec[n] = Integer.parseInt(ySplit[n]);
						}
    					cellOutline[j] = new Polygon(xvec, yvec, npoints);
    					
    					Element trackLabele = (Element)trackLabels.item(j);
    					linkedData[i].trackLabel[j] = Integer.parseInt(trackLabele.getFirstChild().getNodeValue());
    					
    					Element mothere = (Element)mothers.item(j);
    					linkedData[i].mother[j] = Integer.parseInt(mothere.getFirstChild().getNodeValue());
    					
    					Element Nchildrene = (Element)Nchildrens.item(j);
    					int Nchildren = Integer.parseInt(Nchildrene.getFirstChild().getNodeValue());
    					if(Nchildren>0){
    						Element daugterse = (Element)daugters.item(daugtercont);
    						daugtercont++;
        					linkedData[i].children[j].addElement(new Integer(Integer.parseInt(daugterse.getFirstChild().getNodeValue())));
    					}
    				}
    				extractedData[i].setNregions(Nregions);
    				extractedData[i].setRegionCentroidX(regionCentroidX);
    				extractedData[i].setRegionCentroidY(regionCentroidY);
    				extractedData[i].setRegionArea(regionArea);
    				extractedData[i].setCellArea(cellArea);
    				extractedData[i].setRegionXmin(regionXmin);
    				extractedData[i].setRegionXmax(regionXmax);
    				extractedData[i].setRegionYmin(regionYmin);
    				extractedData[i].setRegionYmax(regionYmax);
    				extractedData[i].recomputeRegionHeightFromExtremes();
    				extractedData[i].recomputeRegionWidthFromExtremes();
    				extractedData[i].setRegionMeanCh1(regionMeanCh1);
    				extractedData[i].setRegionMeanCh2(regionMeanCh2);
    				extractedData[i].setMinValueRegionCh1(minValueRegionCh1);
    				extractedData[i].setMinValueRegionCh2(minValueRegionCh2);
    				extractedData[i].setMinValueRegionNotCellCh1(minValueRegionNotCellCh1);
    				extractedData[i].setMinValueRegionNotCellCh2(minValueRegionNotCellCh2);
    				extractedData[i].setRegionBottomNPercentValueCh1(regionBottomNPercentValueCh1);
    				extractedData[i].setRegionBottomNPercentValueCh2(regionBottomNPercentValueCh2);
    				extractedData[i].setRegionNotCellBottomNPercentValueCh1(regionNotCellBottomNPercentValueCh1);
    				extractedData[i].setRegionNotCellBottomNPercentValueCh2(regionNotCellBottomNPercentValueCh2);
					extractedData[i].setRegionNotCellMeanVSCellMeanCh1(regionNotCellMeanVSCellMeanCh1);
    				extractedData[i].setRegionNotCellMeanVSCellMeanCh2(regionNotCellMeanVSCellMeanCh2);
    				extractedData[i].setEnergyLoG10Ch1(energyLoG10Ch1);
    				extractedData[i].setEnergyLoG10Ch2(energyLoG10Ch2);
    				extractedData[i].setEnergyLoG3Ch1(energyLoG3Ch1);
    				extractedData[i].setEnergyLoG3Ch2(energyLoG3Ch2);
    				extractedData[i].setEnergySteerable4Ch1(energySteerable4Ch1);
    				extractedData[i].setEnergySteerable4Ch2(energySteerable4Ch2);
    				extractedData[i].setRegionMeanCh2(regionMeanCh2);
    				extractedData[i].setCellMeanCh1(cellMeanCh1);
    				extractedData[i].setCellMeanCh2(cellMeanCh2);
    				extractedData[i].setCellTop3MeanCh1(cellTop3MeanCh1);
    				extractedData[i].setCellTop3MeanCh2(cellTop3MeanCh2);
    				extractedData[i].setCellTop5MeanCh1(cellTop5MeanCh1);
    				extractedData[i].setCellTop5MeanCh2(cellTop5MeanCh2);
    				extractedData[i].setCellMedianCh1(cellMedianCh1);
    				extractedData[i].setCellMedianCh2(cellMedianCh2);
    				extractedData[i].setSegmentationType(segmentationType);
    				extractedData[i].setSegmentationQuality(segmentationQuality);
    				extractedData[i].setCellCircularity(cellCircularity);
    				extractedData[i].setCellMeanVScellStdCh1(cellMeanVScellStdCh1);
    				extractedData[i].setCellMeanVScellStdCh2(cellMeanVScellStdCh2);
    				extractedData[i].setCellOutline(cellOutline);
                }
	         	runParameters.roiBox = roiBox;
	    		runParameters.lambda = lambda;
	    		runParameters.sigma = sigma;
	    		runParameters.sigmaNotMovingRegion = sigmaNotMovingRegion;
	    		runParameters.regionOffset = regionOffset;
	    		runParameters.downDisplacement = downDisplacement;
	    		runParameters.upDisplacement = upDisplacement;
	    		runParameters.lateralDisplacement = lateralDisplacement;
	    		runParameters.regularizationIter = regularizationIter;
	    	}catch(ParserConfigurationException pce) {
				IJ.error(""+pce);
				return false;
			}catch(SAXException se) {
				IJ.error(""+se);
				return false;
			}catch(IOException ioe) {
				IJ.error(""+ioe);
				return false;
			}
		}else{
			return false;
		}
		
		
		return true;
	}
	
	private static String getTextValue(Element ele, String tagName) {
		String textVal = null;
		NodeList nl = ele.getElementsByTagName(tagName);
		if(nl != null && nl.getLength() > 0) {
			Element el = (Element)nl.item(0);
			textVal = el.getFirstChild().getNodeValue();
		}
		return textVal;
	}
}
