/*====================================================================
| Version: December 7, 2010
\===================================================================*/

/*====================================================================
| Ricard Delgado-Gonzalo
| EPFL/STI/IOA/LIB/BM.4.141
| e-mail: ricard.delgado@gmail.com
| URL: http://www.ricarddelgado.com/
\===================================================================*/
package io;

import ij.IJ;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import structures.CTConnectivity;
import structures.CTExtractedData;
import structures.CTRunParameters;

public class CTOutput {

	public static void saveSingleCellOutline(String outputFile, CTExtractedData[] extractedData) { 
		try {
			BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile+"_singlecelloutline.txt"));

			int nFrames = extractedData.length;
			//find the largest polygon size
			int maxPolygonSize = 0;
			for(int i=0; i<nFrames; i++){
				int Nregions = extractedData[i].getNregions();
				Polygon[] outlinesPolygons = extractedData[i].getCellOutline();
				for(int j=0; j<Nregions; j++){
					if(outlinesPolygons[j].npoints>maxPolygonSize){
						maxPolygonSize = outlinesPolygons[j].npoints;
					}
				}
			}
			
			for(int i=0; i<nFrames; i++){
				int Nregions = extractedData[i].getNregions();
				Polygon[] outlinesPolygons = extractedData[i].getCellOutline();
				        
				for(int j=0; j<Nregions; j++){
					String str = new String(""+i);
					str=str+"\t"+j;
			
					int polygonSize = outlinesPolygons[j].npoints;
					Polygon currentOutline = outlinesPolygons[j];
					
					for(int k=0; k<polygonSize; k++){
						str=str+"\t"+currentOutline.xpoints[k]; 
					}
					for(int k=polygonSize; k<maxPolygonSize; k++){
						str=str+"\t"+"NaN";
					}
					
					for(int k=0; k<polygonSize; k++){
						str=str+"\t"+currentOutline.ypoints[k]; 
					}
					for(int k=polygonSize; k<maxPolygonSize; k++){
						str=str+"\t"+"NaN";
					}
					writer.write(str+"\n");
				}
			}
			writer.flush();
			writer.close();
		} catch (IOException e) {
			IJ.error("IOError: " + e.getMessage());
		}
	}
	
	public static void saveSingleCell(String outputFile, CTExtractedData[] extractedData) { 
		try {
			BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile+"_singlecell.txt"));

			int nFrames = extractedData.length;
			for(int i=0; i<nFrames; i++){
				int Nregions = extractedData[i].getNregions();
				
				double[] regionCentroidX 						= extractedData[i].getRegionCentroidX();
				double[] regionCentroidY 						= extractedData[i].getRegionCentroidY();
				double[] regionArea								= extractedData[i].getRegionArea();
				double[] cellArea 								= extractedData[i].getCellArea();
				double[] cellMeanCh1 							= extractedData[i].getCellMeanCh1();
				double[] cellMeanCh2 							= extractedData[i].getCellMeanCh2();
				double[] cellTop3MeanCh1 						= extractedData[i].getCellTop3MeanCh1();
				double[] cellTop3MeanCh2 						= extractedData[i].getCellTop3MeanCh2();
				double[] cellTop5MeanCh1 						= extractedData[i].getCellTop5MeanCh1();
				double[] cellTop5MeanCh2 						= extractedData[i].getCellTop5MeanCh2();
				double[] cellMedianCh1 							= extractedData[i].getCellMedianCh1();
				double[] cellMedianCh2 							= extractedData[i].getCellMedianCh2();
				double[] regionMeanCh1 							= extractedData[i].getRegionMeanCh1();
				double[] regionMeanCh2 							= extractedData[i].getRegionMeanCh2();
				double[] minValueRegionCh1 						= extractedData[i].getMinValueRegionCh1();
				double[] minValueRegionCh2 						= extractedData[i].getMinValueRegionCh2();
				double[] minValueRegionNotCellCh1 				= extractedData[i].getMinValueRegionNotCellCh1();
				double[] minValueRegionNotCellCh2 				= extractedData[i].getMinValueRegionNotCellCh2();
				double[] regionBottomNPercentValueCh1 			= extractedData[i].getRegionBottomNPercentValueCh1();
				double[] regionBottomNPercentValueCh2 			= extractedData[i].getRegionBottomNPercentValueCh2();
				double[] regionNotCellBottomNPercentValueCh1	= extractedData[i].getRegionNotCellBottomNPercentValueCh1();
				double[] regionNotCellBottomNPercentValueCh2 	= extractedData[i].getRegionNotCellBottomNPercentValueCh2();
				int[] segmentationType							= extractedData[i].getSegmentationType();
				double[] segmentationQuality 					= extractedData[i].getSegmentationQuality();
				double[] cellCircularity						= extractedData[i].getCellCircularity();
				double[] cellMeanVScellStdCh1					= extractedData[i].getCellMeanVScellStdCh1();
				double[] cellMeanVScellStdCh2					= extractedData[i].getCellMeanVScellStdCh2();
				double[] regionNotCellMeanVSCellMeanCh1 		= extractedData[i].getRegionNotCellMeanVSCellMeanCh1();
				double[] regionNotCellMeanVSCellMeanCh2 		= extractedData[i].getRegionNotCellMeanVSCellMeanCh2();
				double[] energyLoG10Ch1							= extractedData[i].getEnergyLoG10Ch1();
				double[] energyLoG10Ch2							= extractedData[i].getEnergyLoG10Ch2();
				double[] energyLoG3Ch1							= extractedData[i].getEnergyLoG3Ch1();
				double[] energyLoG3Ch2							= extractedData[i].getEnergyLoG3Ch2();
				double[] energySteerable4Ch1					= extractedData[i].getEnergySteerable4Ch1();
				double[] energySteerable4Ch2					= extractedData[i].getEnergySteerable4Ch2();
				
				for(int j=0; j<Nregions; j++){
					String str = new String(""+i);
					str=str+"\t"+j;
					str=str+"\t"+regionCentroidX[j];
					str=str+"\t"+regionCentroidY[j];
					str=str+"\t"+regionArea[j];
					str=str+"\t"+regionMeanCh1[j];
					str=str+"\t"+regionMeanCh2[j];
					str=str+"\t"+cellArea[j];
					str=str+"\t"+cellMeanCh1[j];
					str=str+"\t"+cellMeanCh2[j];
					str=str+"\t"+cellTop3MeanCh1[j];
					str=str+"\t"+cellTop3MeanCh2[j];
					str=str+"\t"+cellTop5MeanCh1[j];
					str=str+"\t"+cellTop5MeanCh2[j];
					str=str+"\t"+cellMedianCh1[j];
					str=str+"\t"+cellMedianCh2[j];
					str=str+"\t"+minValueRegionCh1[j];
					str=str+"\t"+minValueRegionCh2[j];
					str=str+"\t"+minValueRegionNotCellCh1[j];
					str=str+"\t"+minValueRegionNotCellCh2[j];
					str=str+"\t"+regionBottomNPercentValueCh1[j];
					str=str+"\t"+regionBottomNPercentValueCh2[j];
					str=str+"\t"+regionNotCellBottomNPercentValueCh1[j];
					str=str+"\t"+regionNotCellBottomNPercentValueCh2[j];
					str=str+"\t"+segmentationType[j];
					str=str+"\t"+segmentationQuality[j];
					str=str+"\t"+cellCircularity[j];
					str=str+"\t"+cellMeanVScellStdCh1[j];
					str=str+"\t"+cellMeanVScellStdCh2[j];
					str=str+"\t"+regionNotCellMeanVSCellMeanCh1[j];
					str=str+"\t"+regionNotCellMeanVSCellMeanCh2[j];
					str=str+"\t"+energyLoG10Ch1[j];
					str=str+"\t"+energyLoG10Ch2[j];
					str=str+"\t"+energyLoG3Ch1[j];
					str=str+"\t"+energyLoG3Ch2[j];
					str=str+"\t"+energySteerable4Ch1[j];
					str=str+"\t"+energySteerable4Ch2[j];
					writer.write(str+"\n");
				}
			}
			writer.flush();
			writer.close();
		} catch (IOException e) {
			IJ.error("IOError: " + e.getMessage());
		}
	}
	
	public static void saveXML(String outputFile, CTRunParameters runParameters, CTExtractedData[] extractedData, CTConnectivity[] linkedData) { 
		try {
			BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile+".xml"));
			
			writer.write("<?xml version=\"1.0\" ?>\n");
			
			writer.write("<results>\n");
			
			Rectangle boundingROI = runParameters.roiBox.getBounds();
			writer.write("<ROI>\n");
				writer.write("<xMin>"+boundingROI.x+"</xMin>\n");
				writer.write("<yMin>"+boundingROI.y+"</yMin>\n");
				writer.write("<W>"+boundingROI.width+"</W>\n");
				writer.write("<H>"+boundingROI.height+"</H>\n");
			writer.write("</ROI>\n");
		
			writer.write("<Nim>"+extractedData.length+"</Nim>\n");
			
			writer.write("<iter>"+runParameters.regularizationIter+"</iter>\n");
			writer.write("<lambda>"+runParameters.lambda+"</lambda>\n");
			writer.write("<sigma>"+runParameters.sigma+"</sigma>\n");
			writer.write("<NMR>"+runParameters.sigmaNotMovingRegion+"</NMR>\n");
			writer.write("<offset>"+runParameters.regionOffset+"</offset>\n");
			
			writer.write("<upD>"+runParameters.upDisplacement+"</upD>\n");
			writer.write("<downD>"+runParameters.downDisplacement+"</downD>\n");
			writer.write("<latD>"+runParameters.lateralDisplacement+"</latD>\n");
			
			for(int i=0; i<extractedData.length; i++){
				writer.write("<frame>\n");
				int Nregions = extractedData[i].getNregions();
				
				writer.write("<Nreg>"+Nregions+"</Nreg>\n");
				
				double[] regionCentroidX 						= extractedData[i].getRegionCentroidX();
				double[] regionCentroidY 						= extractedData[i].getRegionCentroidY();
				double[] regionArea 							= extractedData[i].getRegionArea();
				double[] cellArea 								= extractedData[i].getCellArea();
				int[] regionXmin 								= extractedData[i].getRegionXmin();
				int[] regionXmax 								= extractedData[i].getRegionXmax();
				int[] regionYmin 								= extractedData[i].getRegionYmin();
				int[] regionYmax 								= extractedData[i].getRegionYmax();
				int[] regionWidth 								= extractedData[i].getRegionWidth();
				int[] regionHeight 								= extractedData[i].getRegionHeight();
				double[] cellMeanCh1 							= extractedData[i].getCellMeanCh1();
				double[] cellMeanCh2 							= extractedData[i].getCellMeanCh2();
				double[] cellTop3MeanCh1		 				= extractedData[i].getCellTop3MeanCh1();
				double[] cellTop3MeanCh2 						= extractedData[i].getCellTop3MeanCh2();
				double[] cellTop5MeanCh1 						= extractedData[i].getCellTop5MeanCh1();
				double[] cellTop5MeanCh2 						= extractedData[i].getCellTop5MeanCh2();
				double[] cellMedianCh1 							= extractedData[i].getCellMedianCh1();
				double[] cellMedianCh2 							= extractedData[i].getCellMedianCh2();
				double[] regionMeanCh1 							= extractedData[i].getRegionMeanCh1();
				double[] regionMeanCh2 							= extractedData[i].getRegionMeanCh2();
				double[] minValueRegionCh1 						= extractedData[i].getMinValueRegionCh1();
				double[] minValueRegionCh2 						= extractedData[i].getMinValueRegionCh2();
				double[] minValueRegionNotCellCh1 				= extractedData[i].getMinValueRegionNotCellCh1();
				double[] minValueRegionNotCellCh2 				= extractedData[i].getMinValueRegionNotCellCh2();
				double[] regionBottomNPercentValueCh1 			= extractedData[i].getRegionBottomNPercentValueCh1();
				double[] regionBottomNPercentValueCh2 			= extractedData[i].getRegionBottomNPercentValueCh2();
				double[] regionNotCellBottomNPercentValueCh1	= extractedData[i].getRegionNotCellBottomNPercentValueCh1();
				double[] regionNotCellBottomNPercentValueCh2 	= extractedData[i].getRegionNotCellBottomNPercentValueCh2();
				double[] energyLoG10Ch1							= extractedData[i].getEnergyLoG10Ch1();
				double[] energyLoG10Ch2							= extractedData[i].getEnergyLoG10Ch2();
				double[] energyLoG3Ch1							= extractedData[i].getEnergyLoG3Ch1();
				double[] energyLoG3Ch2							= extractedData[i].getEnergyLoG3Ch2();
				double[] energySteerable4Ch1					= extractedData[i].getEnergySteerable4Ch1();
				double[] energySteerable4Ch2					= extractedData[i].getEnergySteerable4Ch2();
				int[] segmentationType 							= extractedData[i].getSegmentationType();
				double[] segmentationQuality 					= extractedData[i].getSegmentationQuality();
				double[] cellCircularity 						= extractedData[i].getCellCircularity();
				double[] cellMeanVScellStdCh1					= extractedData[i].getCellMeanVScellStdCh1();
				double[] cellMeanVScellStdCh2					= extractedData[i].getCellMeanVScellStdCh2();
				double[] regionNotCellMeanVSCellMeanCh1 		= extractedData[i].getRegionNotCellMeanVSCellMeanCh1();
				double[] regionNotCellMeanVSCellMeanCh2 		= extractedData[i].getRegionNotCellMeanVSCellMeanCh2();
				Polygon[] cellOutline		 					= extractedData[i].getCellOutline();
				int[] trackLabels 								= linkedData[i].trackLabel;
				int[] mother 									= linkedData[i].mother;
				Vector<Integer>[] children 						= linkedData[i].children;
				
				for(int j=0; j<Nregions; j++){
					writer.write("<cell>"+j+"</cell>\n");
					writer.write("<track>"+trackLabels[j]+"</track>\n");
					
					writer.write("<rXc>"+regionCentroidX[j]+"</rXc>\n");
					writer.write("<rYc>"+regionCentroidY[j]+"</rYc>\n");
					
					writer.write("<rArea>"+regionArea[j]+"</rArea>\n");
					
					writer.write("<rXmin>"+regionXmin[j]+"</rXmin>\n");
					writer.write("<rXmax>"+regionXmax[j]+"</rXmax>\n");
					
					writer.write("<rYmin>"+regionYmin[j]+"</rYmin>\n");
					writer.write("<rYmax>"+regionYmax[j]+"</rYmax>\n");
					
					writer.write("<rW>"+regionWidth[j]+"</rW>\n");
					writer.write("<rH>"+regionHeight[j]+"</rH>\n");
					
					writer.write("<rMean1>"+regionMeanCh1[j]+"</rMean1>\n");
					writer.write("<rMean2>"+regionMeanCh2[j]+"</rMean2>\n");
					
					writer.write("<rMin1>"+minValueRegionCh1[j]+"</rMin1>\n");
					writer.write("<rMin2>"+minValueRegionCh2[j]+"</rMin2>\n");
					
					writer.write("<rNotCellMin1>"+minValueRegionNotCellCh1[j]+"</rNotCellMin1>\n");
					writer.write("<rNotCellMin2>"+minValueRegionNotCellCh2[j]+"</rNotCellMin2>\n");

					writer.write("<rBot5per1>"+regionBottomNPercentValueCh1[j]+"</rBot5per1>\n");
					writer.write("<rBot5per2>"+regionBottomNPercentValueCh2[j]+"</rBot5per2>\n");
					
					writer.write("<rNotCellBot5per1>"+regionNotCellBottomNPercentValueCh1[j]+"</rNotCellBot5per1>\n");
					writer.write("<rNotCellBot5per2>"+regionNotCellBottomNPercentValueCh2[j]+"</rNotCellBot5per2>\n");
					
					writer.write("<area>"+cellArea[j]+"</area>\n");
					
					writer.write("<mean1>"+cellMeanCh1[j]+"</mean1>\n");
					writer.write("<mean2>"+cellMeanCh2[j]+"</mean2>\n");
			
					writer.write("<Top3mean1>"+cellTop3MeanCh1[j]+"</Top3mean1>\n");
					writer.write("<Top3mean2>"+cellTop3MeanCh2[j]+"</Top3mean2>\n");
					
					writer.write("<Top5mean1>"+cellTop5MeanCh1[j]+"</Top5mean1>\n");
					writer.write("<Top5mean2>"+cellTop5MeanCh2[j]+"</Top5mean2>\n");
			
					writer.write("<median1>"+cellMedianCh1[j]+"</median1>\n");
					writer.write("<median2>"+cellMedianCh2[j]+"</median2>\n");
					
					writer.write("<meanVSstd1>"+cellMeanVScellStdCh1[j]+"</meanVSstd1>\n");
					writer.write("<meanVSstd2>"+cellMeanVScellStdCh2[j]+"</meanVSstd2>\n");
										
					writer.write("<meanNotCellVSCell1>"+regionNotCellMeanVSCellMeanCh1[j]+"</meanNotCellVSCell1>\n");
					writer.write("<meanNotCellVSCell2>"+regionNotCellMeanVSCellMeanCh2[j]+"</meanNotCellVSCell2>\n");
					
					writer.write("<LoG10ch1>"+energyLoG10Ch1[j]+"</LoG10ch1>\n");
					writer.write("<LoG10ch2>"+energyLoG10Ch2[j]+"</LoG10ch2>\n");

					writer.write("<LoG3ch1>"+energyLoG3Ch1[j]+"</LoG3ch1>\n");
					writer.write("<LoG3ch2>"+energyLoG3Ch2[j]+"</LoG3ch2>\n");
					
					writer.write("<steerable4ch1>"+energySteerable4Ch1[j]+"</steerable4ch1>\n");
					writer.write("<steerable4ch2>"+energySteerable4Ch2[j]+"</steerable4ch2>\n");
					
					writer.write("<outline>\n");
					if(segmentationType[j]==CTExtractedData.OVUSCULE){
						writer.write("<type>OVUSCULE</type>\n");
					}else if(segmentationType[j]==CTExtractedData.ESNAKE){
						writer.write("<type>ESNAKE</type>\n");
					}else if(segmentationType[j]==CTExtractedData.REGION){
						writer.write("<type>REGION</type>\n");
					}
					writer.write("<qos>"+segmentationQuality[j]+"</qos>\n");
					writer.write("<circ>"+cellCircularity[j]+"</circ>\n");
					
					int M = cellOutline[j].npoints;
					writer.write("<Npol>"+M+"</Npol>\n");
					
					String xVec;
					String yVec;
					
					if(M>0){
						xVec=""+cellOutline[j].xpoints[0];
						yVec=""+cellOutline[j].ypoints[0];
					}else{
						xVec = "";
						yVec = "";
					}
					
					for(int n=1; n<M; n++){
						xVec=xVec+"\t"+cellOutline[j].xpoints[n];
						yVec=yVec+"\t"+cellOutline[j].ypoints[n];
					}				

					writer.write("<x>"+xVec+"</x>\n");
					writer.write("<y>"+yVec+"</y>\n");
					writer.write("</outline\n>");
					
					writer.write("<mother>"+mother[j]+"</mother>\n");
					int Nchildren = children[j].size();
					writer.write("<Nch>"+Nchildren+"</Nch>\n");
					for(int k=0; k<Nchildren; k++){
						writer.write("<child>"+children[j].elementAt(k).intValue()+"</child>\n");
					}
				}
				writer.write("</frame>\n");
			}
			writer.write("</results>\n");
			
			writer.flush();
			
			writer.close();
		} catch (IOException e) {
			IJ.error("IOError: " + e.getMessage());
		}
	}
	
}

