/*====================================================================
| Version: December 7, 2010
\===================================================================*/

/*====================================================================
| Ricard Delgado-Gonzalo
| EPFL/STI/IOA/LIB/BM.4.141
| e-mail: ricard.delgado@gmail.com
| URL: http://www.ricarddelgado.com/
\===================================================================*/
package structures;

import java.awt.Color;
import java.awt.Polygon;

import ij.IJ;
import ij.process.FloatProcessor;
import ij.process.ImageProcessor;


public class CTExtractedData {
	
	public static String[] segmentationTypeName = {"Ovuscule", "E-Snake", "Region"};
	public static Color[] segmentationTypeColor = {Color.yellow, Color.red, Color.blue};
	
	public static final int OVUSCULE = 0;
	public static final int ESNAKE   = 1;
	public static final int REGION 	 = 2;
	
	private FloatProcessor regionLabelMap = null;
	private boolean[][] neighborhoodMatrix = null; 
	private double[] regionCentroidX = null; 
	private double[] regionCentroidY = null; 
	private double[] regionArea = null;
	private double[] cellArea = null;
	private int Nregions = 0;
	private int[] regionXmin = null;
	private int[] regionXmax = null;
	private int[] regionYmin = null;
	private int[] regionYmax = null;
	private int[] regionWidth = null;
	private int[] regionHeight = null;
	private double[][] regionImageValuesCh1 = null;
	private double[][] regionImageValuesCh2 = null;
	private double[] cellMeanCh1 = null;
	private double[] cellMeanCh2 = null;
	private double[] cellTop5MeanCh1 = null;
	private double[] cellTop5MeanCh2 = null;
	private double[] cellTop3MeanCh1 = null;
	private double[] cellTop3MeanCh2 = null;
	private double[] cellMedianCh1 = null;
	private double[] cellMedianCh2 = null;
	private double[] regionMeanCh1 = null;
	private double[] regionMeanCh2 = null;
	private double[] minValueRegionCh1 = null;
	private double[] minValueRegionCh2 = null;
	private double[] minValueRegionNotCellCh1 = null;
	private double[] minValueRegionNotCellCh2 = null;
	private double[] regionBottomNPercentValueCh1 = null;
	private double[] regionBottomNPercentValueCh2 = null;
	private double[] regionNotCellBottomNPercentValueCh1 = null;
	private double[] regionNotCellBottomNPercentValueCh2 = null;
	private Polygon[] cellOutline = null;
	private int[] segmentationType = null;
	private double[] segmentationQuality = null;
	private double[] cellCircularity = null;
	private double[] cellMeanVScellStdCh1 = null;
	private double[] cellMeanVScellStdCh2 = null;
	private double[] regionNotCellMeanVSCellMeanCh1 = null;
	private double[] regionNotCellMeanVSCellMeanCh2 = null;
	private double[] energyLoG10Ch1 = null;
	private double[] energyLoG10Ch2 = null;
	private double[] energyLoG3Ch1 = null;
	private double[] energyLoG3Ch2 = null;
	private double[] energySteerable4Ch1 = null;
	private double[] energySteerable4Ch2 = null;
	
	public CTExtractedData(){
	}

	public void printCellInfo(int i) {
		IJ.log("================");
		IJ.log("Cell ID = "+i);
		IJ.log("- segmentation type "+segmentationTypeName[segmentationType[i]]);
		IJ.log("================");

	}
	
	public double[] getEnergySteerable4Ch1(){
		return energySteerable4Ch1;
	}

	public double[] getEnergySteerable4Ch2(){
		return energySteerable4Ch2;
	}

	public void setEnergySteerable4Ch1(double[] energySteerable4Ch1){
		this.energySteerable4Ch1 = energySteerable4Ch1;
	}
	
	public void setEnergySteerable4Ch2(double[] energySteerable4Ch2){
		this.energySteerable4Ch2 = energySteerable4Ch2;
	}
	
	public double[] getEnergyLoG10Ch1(){
		return energyLoG10Ch1;
	}
	
	public double[] getEnergyLoG10Ch2(){
		return energyLoG10Ch2;
	}

	public double[] getEnergyLoG3Ch1(){
		return energyLoG3Ch1;
	}
	
	public double[] getEnergyLoG3Ch2(){
		return energyLoG3Ch2;
	}
	
	public void setEnergyLoG10Ch1(double[] energyLoG10Ch1){
		this.energyLoG10Ch1 = energyLoG10Ch1;
	}

	public void setEnergyLoG10Ch2(double[] energyLoG10Ch2){
		this.energyLoG10Ch2 = energyLoG10Ch2;
	}
		
	public void setEnergyLoG3Ch1(double[] energyLoG3Ch1){
		this.energyLoG3Ch1 = energyLoG3Ch1;
	}

	public void setEnergyLoG3Ch2(double[] energyLoG3Ch2){
		this.energyLoG3Ch2 = energyLoG3Ch2;
	}
	
	public double[] getMinValueRegionCh1(){
		return minValueRegionCh1;
	}

	public double[] getMinValueRegionCh2(){
		return minValueRegionCh2;
	}
	
	public void setMinValueRegionCh1(double[] minValueRegionCh1){
		this.minValueRegionCh1 = minValueRegionCh1;
	}
	
	public void setMinValueRegionCh2(double[] minValueRegionCh2){
		this.minValueRegionCh2 = minValueRegionCh2;
	}
	
	public double[] getMinValueRegionNotCellCh1(){
		return minValueRegionNotCellCh1;
	}

	public double[] getMinValueRegionNotCellCh2(){
		return minValueRegionNotCellCh2;
	}
	
	public void setMinValueRegionNotCellCh1(double[] minValueRegionNotCellCh1){
		this.minValueRegionNotCellCh1 = minValueRegionNotCellCh1;
	}
	
	public void setMinValueRegionNotCellCh2(double[] minValueRegionNotCellCh2){
		this.minValueRegionNotCellCh2 = minValueRegionNotCellCh2;
	}
	
	public double[] getRegionBottomNPercentValueCh1(){
		return regionBottomNPercentValueCh1;
	}

	public double[] getRegionBottomNPercentValueCh2(){
		return regionBottomNPercentValueCh2;
	}
	
	public void setRegionBottomNPercentValueCh1(double[] regionBottomNPercentValueCh1){
		this.regionBottomNPercentValueCh1 = regionBottomNPercentValueCh1;
	}
	
	public void setRegionBottomNPercentValueCh2(double[] regionBottomNPercentValueCh2){
		this.regionBottomNPercentValueCh2 = regionBottomNPercentValueCh2;
	}
	
	public double[] getRegionNotCellBottomNPercentValueCh1(){
		return regionNotCellBottomNPercentValueCh1;
	}

	public double[] getRegionNotCellBottomNPercentValueCh2(){
		return regionNotCellBottomNPercentValueCh2;
	}
	
	public void setRegionNotCellBottomNPercentValueCh1(double[] regionNotCellBottomNPercentValueCh1){
		this.regionNotCellBottomNPercentValueCh1 = regionNotCellBottomNPercentValueCh1;
	}
	
	public void setRegionNotCellBottomNPercentValueCh2(double[] regionNotCellBottomNPercentValueCh2){
		this.regionNotCellBottomNPercentValueCh2 = regionNotCellBottomNPercentValueCh2;
	}
	
	public double[] getCellTop3MeanCh1(){
		return cellTop3MeanCh1;
	}

	public double[] getCellTop3MeanCh2(){
		return cellTop3MeanCh2;
	}
	
	public void setCellTop3MeanCh1(double[] cellTop3MeanCh1){
		this.cellTop3MeanCh1 = cellTop3MeanCh1;
	}

	public void setCellTop3MeanCh2(double[] cellTop3MeanCh2){
		this.cellTop3MeanCh2 = cellTop3MeanCh2;
	}
	
	public double[] getCellTop5MeanCh1(){
		return cellTop5MeanCh1;
	}

	public double[] getCellTop5MeanCh2(){
		return cellTop5MeanCh2;
	}

	public void setCellTop5MeanCh1(double[] cellTop5MeanCh1){
		this.cellTop5MeanCh1 = cellTop5MeanCh1;
	}

	public void setCellTop5MeanCh2(double[] cellTop5MeanCh2){
		this.cellTop5MeanCh2 = cellTop5MeanCh2;
	}
	
	public double[] getCellMedianCh1(){
		return cellMedianCh1;
	}

	public double[] getCellMedianCh2(){
		return cellMedianCh2;
	}

	public void setCellMedianCh1(double[] cellMedianCh1){
		this.cellMedianCh1 = cellMedianCh1;
	}

	public void setCellMedianCh2(double[] cellMedianCh2){
		this.cellMedianCh2 = cellMedianCh2;
	}
	
	public double[] getRegionNotCellMeanVSCellMeanCh1(){
		return regionNotCellMeanVSCellMeanCh1;
	}

	public double[] getRegionNotCellMeanVSCellMeanCh2(){
		return regionNotCellMeanVSCellMeanCh2;
	}
	
	public void setRegionNotCellMeanVSCellMeanCh1(double[] regionNotCellMeanVSCellMeanCh1){
		this.regionNotCellMeanVSCellMeanCh1 = regionNotCellMeanVSCellMeanCh1;
	}
	
	public void setRegionNotCellMeanVSCellMeanCh2(double[] regionNotCellMeanVSCellMeanCh2){
		this.regionNotCellMeanVSCellMeanCh2 = regionNotCellMeanVSCellMeanCh2;
	}
	
	public double[] getCellMeanVScellStdCh1(){
		return cellMeanVScellStdCh1;
	}

	public double[] getCellMeanVScellStdCh2(){
		return cellMeanVScellStdCh2;
	}

	public void setCellMeanVScellStdCh1(double[] cellMeanVScellStdCh1){
		this.cellMeanVScellStdCh1 = cellMeanVScellStdCh1;
	}
	
	public void setCellMeanVScellStdCh2(double[] cellMeanVScellStdCh2){
		this.cellMeanVScellStdCh2 = cellMeanVScellStdCh2;
	}
	
	public double[] getCellCircularity(){
		return cellCircularity;
	}
	
	public void setCellCircularity(double[] cellCircularity){
		this.cellCircularity = cellCircularity;
	}
	
	public double[] getSegmentationQuality(){
		return segmentationQuality;
	}
	
	public double[] getCellArea(){
		return cellArea;
	}
	
	public int[] getSegmentationType(){
		return segmentationType;
	}
	
	public int[] getRegionWidth(){
		return regionWidth;
	}
	
	public int[] getRegionHeight(){
		return regionHeight;
	}

	public double[] getRegionMeanCh1(){
		return regionMeanCh1;
	}

	public double[] getRegionMeanCh2(){
		return regionMeanCh2;
	}
	
	public double[] getCellMeanCh1(){
		return cellMeanCh1;
	}
	
	public double[] getCellMeanCh2(){
		return cellMeanCh2;
	}
	
	public double[][] getRegionImageValuesCh1(){
		return regionImageValuesCh1;
	}

	public double[][] getRegionImageValuesCh2(){
		return regionImageValuesCh2;
	}
	
	public void setRegionLabelMap(FloatProcessor regionLabelMap){
		this.regionLabelMap = regionLabelMap;
	}
	
	public int getNregions(){
		return Nregions;
	}
	
	public FloatProcessor getRegionLabelMap(){
		return regionLabelMap;
	}
	
	public double[] getRegionCentroidX(){
		return regionCentroidX;
	}
	
	public double[] getRegionCentroidY(){
		return regionCentroidY;
	}

	public double[] getRegionArea(){
		return regionArea;
	}
	
	public int[] getRegionXmin(){
		return regionXmin;
	}
	
	public int[] getRegionYmin(){
		return regionYmin;
	}
	
	public int[] getRegionXmax(){
		return regionXmax;
	}
	
	public int[] getRegionYmax(){
		return regionYmax;
	}
	
	public Polygon[] getCellOutline(){
		return cellOutline;
	}
	
	public void setSegmentationQuality(double[] segmentationQuality){
		this.segmentationQuality = segmentationQuality; 
	}
	
	public void setCellArea(double[] cellArea){
		this.cellArea = cellArea;
	}
	
	public void setSegmentationType(int[] segmentationType){
		this.segmentationType = segmentationType;
	}
	
	public void setNeighborhoodMatrix(boolean[][] neighborhoodMatrix){
		this.neighborhoodMatrix = neighborhoodMatrix;
	}
	
	public boolean[][] getNeighborhoodMatrix(){
		return neighborhoodMatrix;
	}
	
	public void setRegionCentroidX(double[] regionCentroidX){
		this.regionCentroidX = regionCentroidX;
	}
	
	public void setRegionCentroidY(double[] regionCentroidY){
		this.regionCentroidY = regionCentroidY;
	}
	
	public void setRegionArea(double[] regionArea){
		this.regionArea = regionArea;
	}
	
	public void setNregions(int Nregions){
		this.Nregions = Nregions;
	}
	
	public void setRegionXmin(int[] regionXmin){
		this.regionXmin = regionXmin;
	}
	
	public void setRegionXmax(int[] regionXmax){
		this.regionXmax = regionXmax;
	}
	
	public void setRegionYmin(int[] regionYmin){
		this.regionYmin = regionYmin;
	}
	
	public void setRegionYmax(int[] regionYmax){
		this.regionYmax = regionYmax;
	}

	public void setRegionMeanCh1(double[] regionMeanCh1) {
		this.regionMeanCh1 = regionMeanCh1;
	}

	public void setRegionMeanCh2(double[] regionMeanCh2) {
		this.regionMeanCh2 = regionMeanCh2;
	}
	
	public void setCellMeanCh1(double[] cellMeanCh1) {
		this.cellMeanCh1 = cellMeanCh1;
	}
	
	public void setCellMeanCh2(double[] cellMeanCh2) {
		this.cellMeanCh2 = cellMeanCh2;
	}
	
	public void setCellOutline(Polygon[] cellOutline) {
		this.cellOutline  = cellOutline;
	}
	
	public void extractRegionImageValuesCh1(ImageProcessor imProcessor) {
		regionImageValuesCh1 = new double[Nregions][];
		regionWidth = new int[Nregions];
		regionHeight = new int[Nregions];
		for(int i=0; i<Nregions; i++){
			regionWidth[i] = regionXmax[i]-regionXmin[i]+1;
			regionHeight[i] = regionYmax[i]-regionYmin[i]+1;
			regionImageValuesCh1[i] = new double[regionHeight[i]*regionWidth[i]];
			for(int x=regionXmin[i]; x<=regionXmax[i]; x++){
				for(int y=regionYmin[i]; y<=regionYmax[i]; y++){
					regionImageValuesCh1[i][x-regionXmin[i] + (y-regionYmin[i])*regionWidth[i]] = (double)imProcessor.getPixelValue(x, y);
					
				}
			}
		}
	}

	public void extractRegionImageValuesCh2(ImageProcessor imProcessor) {
		regionImageValuesCh2 = new double[Nregions][];
		regionWidth = new int[Nregions];
		regionHeight = new int[Nregions];
		for(int i=0; i<Nregions; i++){
			regionWidth[i] = regionXmax[i]-regionXmin[i]+1;
			regionHeight[i] = regionYmax[i]-regionYmin[i]+1;
			regionImageValuesCh2[i] = new double[regionHeight[i]*regionWidth[i]];
			for(int x=regionXmin[i]; x<=regionXmax[i]; x++){
				for(int y=regionYmin[i]; y<=regionYmax[i]; y++){
					regionImageValuesCh2[i][x-regionXmin[i] + (y-regionYmin[i])*regionWidth[i]] = (double)imProcessor.getPixelValue(x, y);
					
				}
			}
		}
	}

	public void clearRegionImageValuesCh1() {
		regionImageValuesCh1 = null;
	}

	public void clearRegionImageValuesCh2() {
		regionImageValuesCh2 = null;
	}

	public void recomputeRegionWidthFromExtremes(){
		regionWidth = new int[Nregions];
		if(regionXmax!=null && regionXmin!=null){
			for(int i=0; i<Nregions; i++){
				regionWidth[i] = regionXmax[i]-regionXmin[i]+1;
			}
		}
	}
	
	public void recomputeRegionHeightFromExtremes(){
		regionHeight = new int[Nregions];
		if(regionYmax!=null && regionYmin!=null){
			for(int i=0; i<Nregions; i++){
				regionHeight[i] = regionYmax[i]-regionYmin[i]+1;
			}
		}
	}
}