/*====================================================================
| Version: November 11, 2010
\===================================================================*/

/*====================================================================
 | Ricard Delgado-Gonzalo
 | EPFL/STI/IOA/LIB/BM.4.141
 | e-mail: ricard.delgado@gmail.com
 | URL: http://www.ricarddelgado.com/
 \===================================================================*/
package tracking;

import java.util.Arrays;
import java.util.Vector;

import structures.CTEdge;

public class CTBellmanFord {
	public static double NOVAL = Double.POSITIVE_INFINITY;

	private Vector<CTEdge> cTEdges;
	private double[] distance;
	private int[] predecessor;
	private int Nnodes;
	private int source;

	public CTBellmanFord(Vector<CTEdge> cTEdges, int Nnodes, int source) {
		this.cTEdges = cTEdges;
		this.Nnodes = Nnodes;
		this.source = source;
	}

	public int[] getPredecessor() {
		return predecessor;
	}

	public double[] getTotalWeight() {
		return distance;
	}

	public void computeSpanningTree() {
		distance = new double[Nnodes];
		predecessor = new int[Nnodes];
		Arrays.fill(distance, NOVAL);
		Arrays.fill(predecessor, -1);
		distance[source] = 0.0;

		for (int i = 0; i < Nnodes; ++i) {
			// relax every edge in 'edges'
			for (int j = 0; j < cTEdges.size(); ++j) {
				if (distance[cTEdges.get(j).source] == NOVAL)
					continue;

				double newDistance = distance[cTEdges.get(j).source]
						+ cTEdges.get(j).weight;

				if (newDistance < distance[cTEdges.get(j).destination]) {
					distance[cTEdges.get(j).destination] = newDistance;
					predecessor[cTEdges.get(j).destination] = cTEdges.get(j).source;
				}
			}
		}
	}
}
